//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Bivariate2DInterpolantScattered.h --- Implementation of Interpolant2D class
//
//	DESCRIPTION:
//		Implementation of Interpolant2D class with bivariate interpolant
//		using rectangular grid points - based on routine TOMS 760
//
//	CHANGE HISTORY:
//		UL, 1999-09-08: 1.0.0: - first version
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/slatec/cxx/Bivariate2DInterpolantRectangular.h,v 1.1 2000/01/25 17:55:24 ulammers Exp $
//

#ifndef BIVARIATE_TWOD_INTERPOLANT_RECTANGULAR_H
#define BIVARIATE_TWOD_INTERPOLANT_RECTANGULAR_H

#include "Interpolant2D.h"

namespace Slatec {

template<class NumericType>
class Bivariate2DInterpolantRectangular :
							public virtual Interpolant2D<NumericType> {
public:
	Bivariate2DInterpolantRectangular(const vector<NumericType> &x,
									  const vector<NumericType> &y,
									  const vector<NumericType> &z);
		//
		//	PURPOSE:
		//		Instantiate a Bivariate2DInterpolantRectangular object
		//
		//	ARGUMENTS:
		//		x/y	: vectors of independent variable values
		//		z	: vector of dependent variable values corresponding to
		//			  <x/y>
		//

	NumericType operator()(NumericType x, NumericType y) const;
		//
		//	PURPOSE:
		//		Evaluate interpolation function at a point <x,y>
		//


	~Bivariate2DInterpolantRectangular();
		//
		//	PURPOSE:
		//		Destroy object instantiated with above constructor(s).
		//

private:
	NumericType	*_xd, *_yd,		//	grid point values ...
			    *_zd,			//	and function values at those points
				*_wk;			//	work array
	int			_nxd, _nyd;		//	size of _xd/_yd arrays; _zd has to be
								//	of size _nxd*_nyd
};

}

#endif
