//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Curve.h --- A class representing a smooth curve
//
//	DESCRIPTION:
//		Instances of class Curve represent 1D-functions which can at least
//		be
//			o evaluated at an abitrary abcissa point
//			o integrated between to points
//
//	CHANGE HISTORY:
//		UL, 2000-03-27: 1.0.0: - first version
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/slatec/cxx/Curve.h,v 1.1 2000/04/06 14:08:51 gvacanti Exp $
//

#ifndef CURVE_H
#define CURVE_H

#include "Tracer.h"

namespace Slatec {

template<class NumericType>
class Curve {
public:
	Curve() {}
	typedef NumericType (*Functor)(NumericType);
	Curve(Functor f) : _f(f) { PROC ENDPROC }
		//
		//	PURPOSE:
		//		Instantiate an Curve object
		//


	virtual NumericType operator()(NumericType x) const {
		PROC
		RETURN (*_f)(x);
	}
		//
		//	PURPOSE:
		//		Evaluate interpolation function at a point <X>
		//


	virtual NumericType integral(NumericType x1, NumericType x2) const;
		//
		//	PURPOSE:
		//		Calculate integral of Curve between <x1> and <x2>
		//


	virtual ~Curve() { PROC ENDPROC }
		//
		//	PURPOSE:
		//		Destroy object instantiated with above constructor(s).
		//

	static void f(NumericType);

protected:
	Functor	_f;
};

}

#endif
