//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	DataVector --- Collection of utility classes related to data vectors
//
//	DESCRIPTION:
//		A small collection of utility classes around data vectors.
//
//	CHANGE HISTORY:
//		UL, 2001-11-05: 1.0.0: - first version
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/slatec/cxx/DataVector.h,v 1.2 2001/11/22 10:07:07 ulammers Exp $
//
#ifndef DATA_VECTORS_H
#define DATA_VECTORS_H

#include <algorithm>
#include <functional>
#include <vector>

namespace DataVector {

template<class T>
bool sorted(const std::vector<T> &v) {
    return !v.empty() ? (std::find_if(v.begin(), v.end(),
                        std::bind2nd(std::less<T>(), v[0])) == v.end()) : true;
}

}

template<class T>
class CorrelatedVectors {
public:
	typedef std::vector<T> BaseType;

	CorrelatedVectors(BaseType &free, BaseType &dependent)
		: _free(&free), _dependent(&dependent) {}

	BaseType &free() { return *_free; }
	BaseType &dependent() { return *_dependent; }

	void supplement(const BaseType &extFree, const BaseType &extDep) {
		BaseType::const_iterator extFreeBeg = extFree.begin(),
								 extFreeEnd = extFree.end(),
								 extDepBeg = extDep.begin();
		for (BaseType::const_iterator i = extFreeBeg; i != extFreeEnd; ++i) {
			unsigned ipos = i-extFreeBeg;
			BaseType::iterator freeBeg = _free->begin(),
							   freeEnd = _free->end(),
							   p = lower_bound(freeBeg, freeEnd, *i);
			unsigned pos = p-freeBeg;
			if (p == freeEnd || *p != *i) {
				_free->insert(p, *i);
				_dependent->insert(_dependent->begin()+pos,
								   *(extDepBeg + ipos));
			} else
				*(_dependent->begin()+pos) = *(extDepBeg + ipos);
		}
	}

private:
	BaseType *_free,
			 *_dependent;
};

#endif
