//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	slatec++.h --- Numerical routines from the SLATEC library in C++
//
//	DESCRIPTION:
//		A namespace Slatec is declared with a set of classes which
//		provide numeric functionality available from the SLATEC library.
//
//	CHANGE HISTORY:
//		GV, 1999-09-08: 1.0.0: - first version; error function added
//		UL, 1999-09-10: 1.1.0: - added CubicSpline/HermiteInterpolant
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/slatec/cxx/slatec++.h,v 1.11 2000/09/28 14:19:36 ulammers Exp $
//

#ifndef SLATEC_CPP_H
#define SLATEC_CPP_H

#include <Tracer.h>
#include <vector>

#include "Curve.h"
#include "CubicSplineInterpolant.h"
#include "CubicHermiteInterpolant.h"
#include "LinearInterpolant.h"
#include "Bivariate2DInterpolantScattered.h"
#include "Bivariate2DInterpolantRectangular.h"

namespace Slatec {


  // Error function and complementary error function.
  double erf(const double x);
  double erfc(const double x);
  
  // FFT
  vector<double> forwardFft(const vector<double> & x);
  vector<double> backwardFft(const vector<double> & x);

  // Convolution
  vector<double> convolve(const vector<double> & signal,
			  const vector<double> & response);


/*    // FFT -- these are provided for completeness. Use the higher level */
/*    // interfaces above if possible. */
/*    void rfftf1(int n, float * c, float * ch, float * wa, int * ifac); */
/*    void rfftb1(int n, float * c, float * ch, float * wa, int * ifac); */
/*    void rffti1(int n, float * wa, int * ifac); */

};  
#endif
