//
//	test CubicHermiteInterpolant
//
//	UL, Wed Sep  8 18:13:42 MET DST 1999
//

#include <iostream>
#include <vector>

#include "slatec++.h"
#include "common.h"

using namespace std;

namespace {
bool isClose(double a, double b)
{
	if (a == 0.)
		return fabs(b) < 1.e-10;

    return fabs(a)/fabs(b)-1. < 1.e-1;
}

double f(double x)
{
	return 3.*x*x*x - 2.*x*x + x - 20.;
}

double F(double x)
{
	return .75*x*x*x*x - 2./3.*x*x*x + .5*x*x - 20.*x;
}

double flin(double x)
{
	return x;
}

double Flin(double x)
{
	return .5*x*x;
}

}

int main()
{
	const int N = 20;

	vector<double>	x(N), y(N);
	for (int n=0; n<N; ++n) {
		x[n] = DBL(n);
		y[n] = f(x[n]);
	}

	//	check constructors
	Slatec::CubicHermiteInterpolant<double> chi2;
	Slatec::CubicHermiteInterpolant<double> chi3(3.);
	Slatec::CubicHermiteInterpolant<double> chi4(chi2);

	Slatec::CubicHermiteInterpolant<double> chi(x, y);
	Slatec::CubicSplineInterpolant<double> csi(x, y);
	Slatec::LinearInterpolant<double> li(x, y);
	cout<<chi<<endl;

	//	supplement
	vector<double> xs(10), ys(10);
	for (unsigned n=0; n<10; ++n) {
		xs[n] = N+n;
		ys[n] = f(xs[n]);
	}

	chi.supplement(xs, ys);
	csi.supplement(xs, ys);

	for (int n=-1; n<15*N; ++n) {
		double x = .1 * DBL(n),
			   y1 = chi(x),
			   y2 = csi(x),
			   y3 = li(x);
		cout<<"f("<<x<<") = "<<f(x)<<" - chi="<<y1<<" - csi="<<y2
			<<" - li="<<y3<<endl;
		assert(isClose(y1, f(x)) && isClose(y2, f(x)));
	}


	//
	//	test integral
	//
	double from = 5, to = 15;
	double sumChi = chi.integral(from, to),
		   sumLi = li.integral(from, to),
		   sum0 = F(to)-F(from);
	cout<<"Intergral ["<<from<<", "<<to<<"]: "<<sumChi<<" (chi) "
											  <<sumLi<<" (li) - correct: "
											  <<sum0<<")"<<endl;
	assert(isClose(sumChi, sum0));
	assert(isClose(sumLi, sum0));

	chi += 10.;
	li += 10.;
	sumChi = chi.integral(from, to);
	sumLi = li.integral(from, to);
	cout<<"Intergral ["<<from<<", "<<to<<"]: "<<sumChi<<" (chi) "
											  <<sumLi<<" (li) - correct: "
											  <<sum0+100.<<")"<<endl;
	assert(isClose(sumChi, sum0 + 100.));
	assert(isClose(sumLi, sum0 + 100.));

	//	more tests with LinearInterpolant
	vector<double> x0(4), y0(4);
	x0[0] = y0[0] = 0.;
	x0[1] = y0[1] = 1.;
	x0[2] = y0[2] = 2.;
	x0[3] = y0[3] = 3.;

	Slatec::LinearInterpolant<double> lin(x0, y0);
	cout<<endl;
	from = 0.;
	to = 1.;
	sum0 = Flin(to) - Flin(from);
	cout<<"Integral ["<<from<<", "<<to<<"]: "<<lin.integral(from, to)
		<<" corr: "<<sum0<<endl;

	from = 0.;
	to = 1.;
	sum0 = Flin(to) - Flin(from);
	cout<<"Integral ["<<from<<", "<<to<<"]: "<<lin.integral(from, to)
		<<" corr: "<<sum0<<endl;

	from = -1.;
	to = 0.;
	sum0 = Flin(to) - Flin(from);
	cout<<"Integral ["<<from<<", "<<to<<"]: "<<lin.integral(from, to)
		<<" corr: "<<sum0<<endl;

	from = .5;
	to = 2.5;
	sum0 = Flin(to) - Flin(from);
	cout<<"Integral ["<<from<<", "<<to<<"]: "<<lin.integral(from, to)
		<<" corr: "<<sum0<<endl;

	from = 4.;
	to = 5.;
	sum0 = Flin(to) - Flin(from);
	cout<<"Integral ["<<from<<", "<<to<<"]: "<<lin.integral(from, to)
		<<" corr: "<<sum0<<endl;

	return 0;
}

