//
//	test Bivariate2DInterpolantscatter
//
//	UL, Wed Jan 12 16:43:52 MET 2000
//

#include <iostream>
#include <vector>
#include <cstdlib>

#include "slatec++.h"
#include "common.h"

namespace {
bool isClose(double a, double b)
{
	if (a == 0.)
		return fabs(b) < 1.e-10;

    return fabs(a)/fabs(b)-1. < 1.e-4;
}

static float surface(float x, float y) { return x+3*y; }

}



int main()
{
	const unsigned N = 50;
	vector<float> x(N*N), y(N*N), z(N*N);

	//	test with scattered grid points
	float xg[N][N], yg[N][N], fg[N][N];

	for (unsigned i=0; i<N; ++i)
		for (unsigned j=0; j<N; ++j) {
			xg[i][j] = FLT(i) + FLT(rand())/RAND_MAX;
			yg[i][j] = FLT(j) + FLT(rand())/RAND_MAX;
			fg[i][j] = surface(xg[i][j], yg[i][j]);
		}

	float *xp = xg[0], *yp = yg[0], *fp = fg[0];
	for (unsigned n=0; n<N*N; ++n) {
		x[n] = xp[n];
		y[n] = yp[n];
		z[n] = fp[n];
	}
	
	Slatec::Bivariate2DInterpolantScattered<float> f(x, y, z);

  	for(unsigned i=0; i<N; i++) {
		//	test 100 points
		double x = 20.*FLT(rand())/RAND_MAX,
			   y = 20.*FLT(rand())/RAND_MAX,
			   z0 = surface(x, y),
			   z1 = f(x, y);

		cout<<"f("<<x<<", "<<y<<")= "<<z0<<" - interpolated: "<<z1<<endl;
		assert(isClose(z0, z1));		
  	}

	return 0;
}

